<?php

namespace Drupal\db_queries\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Class DbQueriesSettingsForm. Settings form for the Db Queries module
 *
 * @package Drupal\db_queries\Form
 */
class DbQueriesSettingsForm extends ConfigFormBase {

  /**
   * {@inheritDoc}
   */
  public function getFormId() {
    return 'db_queries_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['db_queries.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->config('db_queries.settings');

    $form['profiling_enabled']= [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable profiling of requests?'),
      '#description' => $this->t('Enables or disables the profiling of requests. If enabled each request will perform a single upsert in the database table. Don\'t forget that each page load may result in many requests.'),
      '#default_value' => $config->get('profiling_enabled'),
    ];

    $form['config'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Configuration'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    ];

    $form['config']['minimum_count'] = [
      '#type' => 'textfield',
      '#title' => $this->t('The minimum request query count to log'),
      '#default_value' => $config->get('minimum_count') ?? "0",
      '#description' => $this->t('Any request that generates fewer queries than this value will not be logged. If zero then all counts are logged.'),
    ];

    $form['config']['exclude_paths'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Excluded URL paths'),
      '#default_value' => $config->get('exclude_paths') ?? "",
      '#rows' => 5,
      '#description' => $this->t(
        'A list of URL paths for which query counts will not be collected.<br>
      Enter one value per line. e.g. "admin/*". Paths use regex patterns as per @link.<br>',
        [
          '@link' => Link::fromTextAndUrl($this->t('preg_match'), Url::fromUri('http://php.net/preg_match'))
            ->toString(),
        ]
      ),
    ];

    $form['actions']['truncate_submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Clear query count data'),
      '#submit' => ['::truncateSubmit'],
    ];

    $message = $this->t("'Clear query count data' removes all existing query count data from the database table. There is no confirmation and no undo.");

    $form['truncate_description'] = [
      '#markup' => '<div id="edit-profiling-enabled--description" class="form-item__description">' . $message . '</div>', 
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $this->config('db_queries.settings')
      ->set('profiling_enabled', $form_state->getValue('profiling_enabled'))
      ->set('minimum_count', $form_state->getValue('minimum_count'))
      ->set('exclude_paths', $form_state->getValue('exclude_paths'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  public function truncateSubmit(array &$form, FormStateInterface $form_state) {

    \Drupal::database()->truncate('db_queries_log')->execute();

    $this->messenger()->addStatus(t('All request profiling data has been deleted.'));
  }
}
